<?php

namespace Drupal\social_event_invite_flow\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Routing\RouteMatchInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\StringTranslation\TranslationInterface;
use Drupal\node\NodeInterface;
use Drupal\user\UserInterface;
use Drupal\Component\Utility\Crypt;
use Drupal\social_event_invite_flow\EventInviteSettingsInterface;
use Drupal\social_event_invite_flow\EventInviteMessageInterface;
use Drupal\social_event\EventEnrollmentInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Component\Utility\EmailValidatorInterface;
use Drupal\Core\Utility\Token;
use Drupal\Component\Render\PlainTextOutput;
use Drupal\Core\Render\Markup;
use Drupal\Core\Url;
use Drupal\Core\Link;


/**
 * Service description.
 */
class EventInviteFlowService {

  use StringTranslationTrait;

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current route match.
   *
   * @var \Drupal\Core\Routing\RouteMatchInterface
   */
  protected $routeMatch;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $account;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected $languageManager;

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected $mailManager;  

  /**
   * The Email validator service.
   *
   * @var \Drupal\Component\Utility\EmailValidatorInterface
   */
  protected $emailValidator;  

  /**
   * The Token service.
   *
   * @var \Drupal\Core\Utility\Token
   */
  protected $token;  

  /**
   * Constructs a Common object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Routing\RouteMatchInterface $route_match
   *   The current route match.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The current user.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(
    EntityTypeManagerInterface $entity_type_manager, 
    ConfigFactoryInterface $config_factory, 
    RouteMatchInterface $route_match, 
    AccountInterface $account, 
    LanguageManagerInterface $language_manager, 
    MailManagerInterface $mail_manager, 
    EmailValidatorInterface $email_validator,
    Token $token) {
      
    $this->entityTypeManager = $entity_type_manager;
    $this->configFactory = $config_factory;
    $this->routeMatch = $route_match;
    $this->account = $account;
    $this->languageManager = $language_manager;
    $this->mailManager = $mail_manager;
    $this->emailValidator = $email_validator;
    $this->token = $token;
  
  }

  /**
   * Gets the default invite messages
   * 
   * @return array
   *   An array of configuration object names.
   */
  public function getEventInviteMessagesDefault() {
    return $this->configFactory->get('social_event_invite_flow.settings');
  }

  public function getEventInviteDefaultMessages() {

    $defaults = FALSE;
   
    $config = $this->configFactory->get('social_event_invite_flow.settings');

    if ($config) {
      $defaults = [
        'reply_to' => $config->get('reply_to'),
        'guests_invite_subject' => $config->get('guests_invite_subject'),
        'guests_invite_message' => ['value' => $config->get('guests_invite_message')],
        'new_accounts_invite_subject' => $config->get('new_accounts_invite_subject'),
        'new_accounts_invite_message' => ['value' => $config->get('new_accounts_invite_message')],
        'existing_accounts_invite_subject' => $config->get('existing_accounts_invite_subject'),
        'existing_accounts_invite_message' => ['value' => $config->get('existing_accounts_invite_message')],
      ];
    }

    return $defaults;

  }


  public function getEventInviteSettings(int $nid) {

    $entity = FALSE;

    $node = $this->entityTypeManager->getStorage('node')->load($nid);

    $storage = $this->entityTypeManager->getStorage('event_invite_settings');
    $event_invite_settings = $storage->loadByProperties(['node' => $node->id()]);

    if ($event_invite_setting = reset($event_invite_settings)) {
      $entity = $event_invite_setting;
    }

    return $entity;


  }  

  public function getInviteSettingsDefaultData() {

    $data = [];

    $config = $this->configFactory->get('social_event_invite_flow.settings');

    /** @var \Drupal\node\Entity\Node $node */
    $node = $this->routeMatch->getParameter('node');

    if (!$node instanceof NodeInterface) {
      $storage = $this->entityTypeManager->getStorage('node');
      $node = $storage->loadByProperties(['node' => $node->id()]);
    }

    // Per default we need to have our config data available.
    $data = [
      'invite_mode_existing_accounts' => $config->get('invite_mode_existing_accounts'),
      'invite_mode_new_accounts' => $config->get('invite_mode_new_accounts'),
      'selected_webform_existing_accounts' => $config->get('selected_webform_existing_accounts'),
      'selected_webform_new_accounts' => $config->get('selected_webform_new_accounts'),
    ];    

    if ($node instanceof NodeInterface) {
      $storage = $this->entityTypeManager->getStorage('event_invite_settings');
      $event_invite_settings = $storage->loadByProperties(['node' => $node->id()]);
      if ($event_invite_setting = reset($event_invite_settings)) {
        $data = [
          'invite_mode_existing_accounts' => $event_invite_setting->getInviteModeExistingAccounts(),
          'invite_mode_new_accounts' => $event_invite_setting->getInviteModeNewAccounts(),
          'selected_webform_existing_accounts' => $event_invite_setting->getSelectedWebformExistingAccounts(),
          'selected_webform_new_accounts' => $event_invite_setting->getSelectedWebformNewAccounts(),
        ];
      }
    }

    // The selectable webforms live only in config,
    // so we added here.
    //$data['webforms_existing_accounts'] = $config->get('webforms_existing_accounts');
    //$data['webforms_new_accounts'] = $config->get('webforms_new_accounts');

    
    return $data;    

  }

  public function saveEventInviteSettings($fields) {

    /** @var \Drupal\node\Entity\Node $node */
    $node = $this->routeMatch->getParameter('node');

    if (!$node instanceof NodeInterface) {
      $storage = $this->entityTypeManager->getStorage('node');
      $node = $storage->loadByProperties(['node' => $node->id()]);
    }   

    //\Drupal::logger('debug')->debug('<pre><code>' . print_r($fields, TRUE) . '</code></pre>');

    $storage = $this->entityTypeManager->getStorage('event_invite_settings');
    $event_invite_settings = $storage->loadByProperties(['node' => $node->id()]);
    if ($event_invite_setting = reset($event_invite_settings)) {
      $event_invite_setting->setInviteModeExistingAccounts($fields['invite_mode_existing_accounts']);
      $event_invite_setting->setInviteModeNewAccounts($fields['invite_mode_new_accounts']);
      $event_invite_setting->setSelectedWebformExistingAccounts($fields['selected_webform_existing_accounts']);
      $event_invite_setting->setSelectedWebformNewAccounts($fields['selected_webform_new_accounts']);
      $event_invite_setting->save();
    }
    else {
      // When creating new entity, we need to add the
      // id field, access token and node field, which are only needed
      // when creating new entities.
      $fields['id'] = $node->bundle() . '_' . $node->uuid();
      $fields['access_token'] = $this->generateAccessToken();
      $fields['node'] = $node->id();
      $new_entity = $storage->create($fields);
      $new_entity->save();
    }

  }

  protected function getInviteMessagesData() {

    $data = [];

    $config = $this->configFactory->get('social_event_invite_flow.settings');

    /** @var \Drupal\node\Entity\Node $node */
    $node = $this->routeMatch->getParameter('node');

    $storage = $this->entityTypeManager->getStorage('event_invite_message');
    $event_invite_messages = $storage->loadByProperties(['node' => $node->id()]);
    if ($event_invite_message = reset($event_invite_messages)) {
      $data = [
        'guests_invite_subject' => $event_invite_message->getGuestsInviteSubject(),
        'guests_invite_message' => $event_invite_message->getGuestsInviteMessage(),
        'new_accounts_invite_subject' => $event_invite_message->getNewAccountsInviteSubject(),
        'new_accounts_invite_message' => $event_invite_message->getNewAccountsInviteMessage(),
        'existing_accounts_invite_subject' => $event_invite_message->getExistingAccountsInviteSubject(),
        'existing_accounts_invite_message' => $event_invite_message->getExistingAccountsInviteMessage(),
      ];
    }
    else {
      $data = [
        'guests_invite_subject' => $config->get('guests_invite_subject'),
        'guests_invite_message' => $config->get('guests_invite_message'),
        'new_accounts_invite_subject' => $config->get('new_accounts_invite_subject'),
        'new_accounts_invite_message' => $config->get('new_accounts_invite_message'),
        'existing_accounts_invite_subject' => $config->get('existing_accounts_invite_subject'),
        'existing_accounts_invite_message' => $config->get('existing_accounts_invite_message'),
      ];
    }

    return $data;

  }

  public function getEventInviteMessageFormElements() {

    $elements = [
      'guests_invite_subject' => [
        'type' => 'textfield',
        'title' => $this->t('Subject for guests'),
        'required' => TRUE,
        'default_value' => $this->getInviteMessagesData()['guests_invite_subject']
      ],
      'guests_invite_message' => [
        'type' => 'text_format',
        'title' => $this->t('Message for guests'),
        'required' => TRUE,
        'default_value' => $this->getInviteMessagesData()['guests_invite_message']
      ],
      'new_accounts_invite_subject' => [
        'type' => 'textfield',
        'title' => $this->t('Subject for new accounts'),
        'required' => TRUE,
        'default_value' => $this->getInviteMessagesData()['new_accounts_invite_subject']        
      ],
      'new_accounts_invite_message' => [
        'type' => 'text_format',
        'title' => $this->t('Message for new accounts'),
        'required' => TRUE,
        'default_value' => $this->getInviteMessagesData()['new_accounts_invite_message']           
      ],
      'existing_accounts_invite_subject' => [
        'type' => 'textfield',
        'title' => $this->t('Subject for existing accounts'),
        'required' => TRUE,
        'default_value' => $this->getInviteMessagesData()['existing_accounts_invite_subject']            
      ],
      'existing_accounts_invite_message' => [
        'type' => 'text_format',
        'title' => $this->t('Message for existing accounts'),
        'required' => TRUE,
        'default_value' => $this->getInviteMessagesData()['existing_accounts_invite_message']            
      ],            
    ];   
    
    return $elements;

  }

  /**
   * Gets an event invite message based on 
   * the given route.
   */
  public function getEventInviteMessages() {

    $entity = FALSE;

    /** @var \Drupal\node\Entity\Node $node */
    $node = $this->routeMatch->getParameter('node');

    if (!$node instanceof NodeInterface) {
      $node = $this->entityTypeManager->getStorage('node')->load($node);
    }    

    $storage = $this->entityTypeManager->getStorage('event_invite_message');
    $event_invite_messages = $storage->loadByProperties(['node' => $node->id()]);

    if ($event_invite_message = reset($event_invite_messages)) {
      $entity = $event_invite_message;
    }

    return $entity;

  }

  public function createEventInviteMessages(array $fields) {

    /** @var \Drupal\node\Entity\Node $node */
    $node = $this->routeMatch->getParameter('node');

    $storage = $this->entityTypeManager->getStorage('event_invite_message');

    $storage->create($fields);
    $entity = $storage->save();

    return $entity;

  }

  public function getAllWebforms() {

    $webforms = FALSE;

    // Get webforms
    $webform_ids = \Drupal::entityQuery('webform')
      ->accessCheck(TRUE)
      ->execute();    
 
    /** @var \Drupal\webform\Entity\Webform $webform */
    foreach ($webform_ids as $webform) {
      $storage = $this->entityTypeManager->getStorage('webform');
      $loaded_webform = $storage->load($webform);
      $webform_machine_name = $loaded_webform->id(); 
      $webform_label = $loaded_webform->label();
      $webforms[$webform_machine_name] = $webform_label;
    }

    return $webforms;

  }

  public function getWebformsExistingAccountsOptions() {

    $webforms = [];

    $social_event_invite_flow = $this->configFactory->get('social_event_invite_flow.settings');
    $webforms = $social_event_invite_flow->get('webforms_existing_accounts');

    return $webforms; 

  }

  public function getWebformsNewAccountsOptions() {

    $webforms = [];

    $social_event_invite_flow = $this->configFactory->get('social_event_invite_flow.settings');
    $webforms = $social_event_invite_flow->get('webforms_new_accounts');

    return $webforms; 

  }  

 

  public function getAvailableFields() {

    $node_field_definitions = [];
    $available_fields = []; 
    $node_bundles = [];
    $profile_bundles = []; 
    $user_bundles = ['user'];
    $forbidden_node_fields = [
      'type',
      'field_group_allowed_join_method',
      'field_group_allowed_visibility'
    ]; 
    $forbidden_profile_fields = [
      'type'
    ];
    $forbidden_user_fields = [
      'init',
      'name'
    ];

    $default_tokens = [
      '[node:title]' => '[node:title]',     
      '[current-user:display-name]' => '[current-user:display-name]',
      '[user:display-name]' => '[user:display-name]',
      '[site:name]' => '[site:name]'
    ];
      


    $available_node_fields = $this->getFields('node', ['event'], $forbidden_node_fields);
   

    // Get profile bundles
    $profile_bundles = $this->getBundle('profile');

    // Default Tokens for user profiles
    $default_tokens_user_profile = $this->getUserProfileBundleTokens($profile_bundles);

    if (isset($profile_bundles) && !empty($profile_bundles)) {

      $available_profile_fields = $this->getFields('profile',$profile_bundles,$forbidden_profile_fields);


    }

    $available_user_fields = $this->getFields('user',$user_bundles,$forbidden_user_fields);

    $available_fields = array_merge($default_tokens,
                                    $default_tokens_user_profile,
                                    $available_node_fields, 
                                    $available_profile_fields,
                                    $available_user_fields);


      
    // Get user field definitions
    
    return $available_fields;

  }

  protected function getUserProfileBundleTokens($profile_bundles) {

    $token_fields = [];

    foreach ($profile_bundles as $bundle_key => $bundle) {
      $token_fields['[user:' . $bundle_key .']'] = '[user:' . $bundle_key . ']';
    }

    return $token_fields;

  }

  protected function getFields(string $entity_type, array $bundles, array $forbidden_fields) {

    $field_definitions = [];
    $available_fields = [];
    


    foreach ($bundles as $bundle_key => $bundle) {

      $field_definitions = \Drupal::service('entity_field.manager')->getFieldDefinitions($entity_type, $bundle_key);

      foreach ($field_definitions as $field_key => $field_value) {

        $field_token_key = strtolower($field_definitions[$field_key]->getName());
        $field_token_key = str_replace(' ','_',$field_token_key);
            
        if ($field_definitions[$field_key]->isRequired() && !in_array($field_key, $forbidden_fields)) {
          if ($entity_type == 'profile') {
            $available_fields['[user:' . $bundle_key . ':' . $field_token_key . ']'] = '[user:' . $bundle_key . ':' . $field_token_key . ']';
          }
          $available_fields['[' . $entity_type . ':' . $field_token_key . ']'] = '[' . $entity_type . ':' . $field_token_key . ']'; 
        }
            
      }

    }

    return $available_fields;

  }

  protected function getBundle($entity_type) {

    $bundles = \Drupal::service('entity_type.bundle.info')->getBundleInfo($entity_type);
    return $bundles;    

  } 

  public function getInviteModeExistingAccountsOptions() {
    return [
      EventInviteSettingsInterface::InviteModeExistingAccountsDefaultEnrollment => $this->t('Default Enrollment'),
      EventInviteSettingsInterface::InviteModeExistingAccountsWebformEnrollment => $this->t('Webform Enrollment')
    ];
  }

  public function getInviteModeNewAccountsOptions() {
    return [
      EventInviteSettingsInterface::InviteModeNewAccountsGuestEnrollment => $this->t('Guest Enrollment'),
      EventInviteSettingsInterface::InviteModeNewAccountsWebformEnrollment => $this->t('Webform Enrollment')
    ];
  }
  
  public function generateAccessToken() {

    $access_token = Crypt::randomBytesBase64();
    return $access_token;

  }

  protected function enrollmentExists($user, $nid) {

    $enrollment_exists = FALSE;

    $conditions = [
      'field_account' => $user->id(),
      'field_event' => $nid,
    ];

    $storage = $this->entityTypeManager->getStorage('event_enrollment');
    $event_enrollment = $storage->loadByProperties($conditions);
    if (!empty($event_enrollment)) {
      $enrollment_exists = TRUE;
    }

    return $enrollment_exists;

  }

  protected function enrollmentExistsByEmail($email, $nid) {

    $enrollment_exists = FALSE;

    $conditions = [
      'field_email' => $email,
      'field_event' => $nid,
    ];

    $storage = $this->entityTypeManager->getStorage('event_enrollment');
    $event_enrollment = $storage->loadByProperties($conditions);
    if (!empty($event_enrollment)) {
      $enrollment_exists = TRUE;
    }

    return $enrollment_exists;

  }
  



  public function enrollExistingAccount($user, $nid) {

    $invite_flow = FALSE;

    // First we need to know which flow we are using
    $storage = $this->entityTypeManager->getStorage('event_invite_settings');
    $event_invite_settings = $storage->loadByProperties(['node' => $nid]);
    if ($event_invite_setting = reset($event_invite_settings)) {
      $invite_flow = $event_invite_setting->getInviteModeExistingAccounts();
    }
      
    if ($invite_flow === EventInviteSettingsInterface::InviteModeExistingAccountsDefaultEnrollment) {
      if (!$this->enrollmentExists($user, $nid)) {
        $fields = [
          'field_event' => $nid,
          'field_enrollment_status' => '1',
          'user_id' => $user->id(),
          'field_account' => $user->id(),
        ];
        $storage = $this->entityTypeManager->getStorage('event_enrollment');
        $new_enrollment = $storage->create($fields);
        $new_enrollment->save();
        $account_enrolled = $new_enrollment;

        // Send emails for enrolled users only
        if ($account_enrolled instanceof EventEnrollmentInterface) {
          $this->sendInviteEmails(
            EventInviteSettingsInterface::InviteModeExistingAccountsDefaultEnrollment,
            $user,
            $nid
          );

        }
      }
    }

    if ($invite_flow === EventInviteSettingsInterface::InviteModeExistingAccountsWebformEnrollment) {
      // Here we just send out the emails the rest will be done
      // by the webform.
      $this->sendInviteEmails(
        EventInviteSettingsInterface::InviteModeExistingAccountsWebformEnrollment,
        $user,
        $nid
      );
    }

  }

  public function enrollNewAccount($email, $nid) {

    $invite_flow = FALSE;
    $invite_access_token = FALSE;
    
    $storage = $this->entityTypeManager->getStorage('event_invite_settings');
    $event_invite_settings = $storage->loadByProperties(['node' => $nid]);
    if ($event_invite_setting = reset($event_invite_settings)) {
      $invite_flow = $event_invite_setting->getInviteModeNewAccounts();
      $invite_access_token = $event_invite_setting->getAccessToken();
    }
    
    if ($invite_flow === EventInviteSettingsInterface::InviteModeNewAccountsGuestEnrollment) {
      // Here we just send out the emails the rest will be done
      // by the user himself (Auto enrollment).
      $this->sendInviteEmails(
        EventInviteSettingsInterface::InviteModeNewAccountsGuestEnrollment,
        $email,
        $nid
      );
    }

    if ($invite_flow === EventInviteSettingsInterface::InviteModeNewAccountsWebformEnrollment) {

      $token = FALSE;
      $storage = $this->entityTypeManager->getStorage('event_enrollment');

      // Create guest enrollment and send emails with token
      if(!$this->enrollmentExistsByEmail($email, $nid)) {
        $generated_token = Crypt::randomBytesBase64();
        $fields = [
          'field_event' => $nid,
          'field_enrollment_status' => '0',          
          'field_email' => $email,
          'field_token' => $generated_token,
        ];        
        $new_enrollment = $storage->create($fields);
        $new_enrollment->save();
        $email_enrolled = $new_enrollment; 
        if ($email_enrolled instanceof EventEnrollmentInterface) {
          $token = $email_enrolled->field_token->value;
        }
      }
      else {
        $existing_enrollments = $storage->loadByProperties([
          'field_event' => $nid,
          'field_token' => $token
        ]);
        if ($existing_enrollment = reset($existing_enrollments)) {
          $token = $existing_enrollment->field_token->value;
        }
      }

      if ($token) {

        $this->sendInviteEmails(
          EventInviteSettingsInterface::InviteModeNewAccountsWebformEnrollment,
          $email,
          $nid,
          $token
        );
      }
    }

    
  }

  protected function sendInviteEmails(string $invite_flow, $attendee, int $nid, $token = FALSE ) {

    $email_sent = FALSE;

    \Drupal::logger('debug')->debug($attendee);

    // Get event invite message for specific invite flow
    $event_invite_message = $this->getInviteMessageFromNid($nid, $invite_flow);


    // Load the node
    $node = $this->entityTypeManager->getStorage('node')->load($nid);

    // Build token context
    $token_context = [];

    if ($node instanceof NodeInterface) {
      $token_context['node'] = $node;
    }

    if ($attendee instanceof UserInterface) {
      $user_mail = $attendee->getEmail();
      $token_context['user'] = $attendee;
      $langcode = $attendee->getPreferredLangcode();
    }
    else {
      $token_context['user'] = $this->account;
      $user_mail = $attendee;
      $langcode = $this->languageManager->getCurrentLanguage()->getId();
    }

   


    if (!$event_invite_message) {
      return;      
    }

    $subject =  PlainTextOutput::renderFromHtml($this->token->replace($event_invite_message['subject'], $token_context));
    $message = $this->token->replace($event_invite_message['message'], $token_context);
    // As we have our action buttons pre-defined we attach it here
    $action_links = [
      '#theme' => 'social_event_invite_flow__actions',
      '#links' => $this->getInviteActionsFromNid($nid, $invite_flow, $token),
    ];
    
    $actions = \Drupal::service('renderer')->render($action_links);

    $message  = Markup::create($message);
    // Attach actions to message
    $message .= $message_text1 . $actions;

    $reply_to = $event_invite_message['reply_to'];

    $context = [
      'subject' => $subject,
      'message' => $message,
    ];    

    // Sending Email
    $delivered = $this->mailManager->mail('system', 'action_send_email', $user_mail, $langcode, ['context' => $context], $reply_to);

    if ($delivered) {
      $email_sent = TRUE;
    }

    return $email_sent;

  }

  protected function getInviteActionsFromNid(int $nid, string $invite_flow, $token = FALSE) {
    
    $links = [];

    // Get our event settings
    $event_invite_setting = $this->getEventInviteSettings($nid);

    // Define link attributes
    $link_options = [
      'absolute' => TRUE,
      'attributes' => [
        'class' => [
          'btn-link', 'btn-link-bg', 'btn-link-one'
        ]
      ]
    ];

    // Get the site name from config
    $config = $this->configFactory->get('system.site');
    $site_name = $config->get('name');

    // Get site url
    $site_url = Url::fromRoute('<front>');
    $site_url->setOptions($link_options);
    $site_link_title = $this->t('About') . ' ' . $site_name; 
    $site_link = Link::fromTextAndUrl($site_link_title, $site_url);

    if ($invite_flow === EventInviteSettingsInterface::InviteModeExistingAccountsDefaultEnrollment) {
      // Add link to event
      $event_url = Url::fromRoute('entity.node.canonical', ['node' => $nid]);
      $event_url->setOptions($link_options);
      $event_link = Link::fromTextAndUrl($this->t('See event'), $event_url);
      $links['event'] = $event_link->toRenderable();
    }

    if ($invite_flow === EventInviteSettingsInterface::InviteModeNewAccountsGuestEnrollment) {
      // First we need to get our token      
      $access_token = $event_invite_setting->getAccessToken();

      // Add link to event
      $event_url = Url::fromRoute('social_event_invite_flow.event_guest_access', ['node' => $nid]);
      // Attach the access token to the url
      $link_options['query'] = ['access_token' => $access_token]; 
      $event_url->setOptions($link_options);     
      $event_link = Link::fromTextAndUrl($this->t('See event'), $event_url);
      $links['event'] = $event_link->toRenderable();
    }

    if ($invite_flow === EventInviteSettingsInterface::InviteModeExistingAccountsWebformEnrollment) {
      // First we need to get the webfrom
      $webform_id = $event_invite_setting->getSelectedWebformExistingAccounts();
      // Add link to event
      $event_url = Url::fromRoute('entity.webform.canonical', ['webform' => $webform_id]);
      // Add query parameter event
      $link_options['query'] = ['event' => $nid];
      $event_url->setOptions($link_options); 
      $event_link = Link::fromTextAndUrl($this->t('See event'), $event_url);
      $links['event'] = $event_link->toRenderable();
    }

    if ($invite_flow === EventInviteSettingsInterface::InviteModeNewAccountsWebformEnrollment) {
      // First we need to get the webfrom
      $webform_id = $event_invite_setting->getSelectedWebformNewAccounts();
      $event_url = Url::fromRoute('entity.webform.canonical', ['webform' => $webform_id]);
      // Add query parameter webform_redirect
      $link_options['query'] = [
        'webform_redirect' => '/node/' . $nid,
        'token' => $token        
      ];
      $event_url->setOptions($link_options);
      $event_link = Link::fromTextAndUrl($this->t('See event'), $event_url);
      $links['event'] = $event_link->toRenderable();
    }

    // Always add the site link 
    $links['site'] = $site_link->toRenderable();

    return $links;

  }

  
  protected function getInviteMessageFromNid(int $nid, string $invite_flow) {

    $message_data = [];

    $message_data['reply_to'] = NULL;

    $storage = $this->entityTypeManager->getStorage('event_invite_message');
    $event_invite_messages = $storage->loadByProperties(['node' => $nid]);

    if ($entity = reset($event_invite_messages)) {
      $event_invite_message = $entity;
    }
 
    // We need to check the invite_flow to get proper message
    if ($invite_flow === EventInviteSettingsInterface::InviteModeExistingAccountsDefaultEnrollment ||
        $invite_flow === EventInviteSettingsInterface::InviteModeExistingAccountsWebformEnrollment) {
      $message_data['subject'] = $event_invite_message->getExistingAccountsInviteSubject();
      $message_data['message'] = $event_invite_message->getExistingAccountsInviteMessage()['value'];
    }
    elseif ($invite_flow === EventInviteSettingsInterface::InviteModeNewAccountsGuestEnrollment) {
      $message_data['subject'] = $event_invite_message->getGuestsInviteSubject();
      $message_data['message'] = $event_invite_message->getGuestsInviteMessage()['value'];
    }
    elseif ($invite_flow === EventInviteSettingsInterface::InviteModeNewAccountsWebformEnrollment) {
      $message_data['subject'] = $event_invite_message->getNewAccountsInviteSubject();
      $message_data['message'] = $event_invite_message->getNewAccountsInviteMessage()['value'];
    }

    if (empty($message_data)) {
      return FALSE;
    }

    // Get Reply to email or use site mail
    $reply_to = $event_invite_message->getReplyTo();
    if (!empty($reply_to)) {
      $message_data['reply_to'] = $reply_to;
    }
  
    return $message_data;

  }

  protected function sendInviteEmail() {
    
  }

  public function addFieldTokens(array &$form) {

    $tokens = [
      'node_title' => '[node:title]',     
      'current_user_display_name' => '[current-user:display-name]',
      'user_display_name' => '[user:display-name]',
      'site_name' => '[site:name]'
    ];    

    $form['tokens'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Available Tokens'),
      '#collapsed' => TRUE,
      '#collapsible' => TRUE  
    ];

    foreach($tokens as $key => $token) {
      $key_value = "'" . $key . "'";
      $link = '<a class="btn btn-sm btn-default" onclick="myFunction(' . $key_value . ')">Copy</a>';

      $form['tokens'][$key]['container'] = [
        '#type' => 'container',
        '#attributes' => ['class' => ['invite--flow']],
      ];

      $form['tokens'][$key]['container']['text'] = [
        '#attributes' => ['id' => $key],
        '#type' => 'textfield',
        '#value' =>  $token,
      ];
      $form['tokens'][$key]['container']['link'] = [
        '#markup' => Markup::Create($link),
        '#prefix' => '<div>',
        '#suffix' => '</div>'
      ];
    }

  }


}
